
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QDragEnterEvent>
#include <QDropEvent>
#include <QRegularExpression>
#include <QProcess>
#include <QThreadPool>
#include "downloadtask.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    process(nullptr),
    trayIcon(nullptr)
{
    ui->setupUi(this);
}

MainWindow::~MainWindow() {
    delete ui;
}

void MainWindow::on_fetchFormats_clicked() {
    currentUrl = ui->urlLineEdit->text().trimmed();
    if (currentUrl.isEmpty()) return;

    process = new QProcess(this);
    connect(process, &QProcess::readyReadStandardOutput, this, &MainWindow::on_processOutput);
    process->start("yt-dlp", QStringList() << "-F" << currentUrl);
}

void MainWindow::on_processOutput() {
    QString output = process->readAllStandardOutput();
    QStringList lines = output.split("
");
    for (QString line : lines) {
        QRegularExpression re(R"((\d+)\s+(\w+).*?(\d+p|audio only))");
        QRegularExpressionMatch match = re.match(line);
        if (match.hasMatch()) {
            QString id = match.captured(1);
            QString fmt = match.captured(2);
            QString quality = match.captured(3);
            ui->formatComboBox->addItem(quality + " [" + fmt + "]", id);
        }
    }
}

void MainWindow::on_downloadVideo_clicked() {
    QString formatCode = ui->formatComboBox->currentData().toString();
    if (formatCode.isEmpty()) return;

    auto *task = new DownloadTask(currentUrl, formatCode);
    connect(task, &DownloadTask::downloadFinished, this, [this](const QString &u){
        showTrayMessage("اكتمل التحميل", u);
    });
    connect(task, &DownloadTask::downloadError, this, [this](const QString &u, const QString &err){
        showTrayMessage("خطأ أثناء التحميل", err);
    });

    QThreadPool::globalInstance()->start(task);
}

void MainWindow::showTrayMessage(const QString &title, const QString &message) {
    if (!trayIcon) {
        trayIcon = new QSystemTrayIcon(this);
        trayIcon->setIcon(QIcon(":/icons/logo.png"));
        trayIcon->show();
    }
    trayIcon->showMessage(title, message, QSystemTrayIcon::Information, 3000);
}

void MainWindow::dragEnterEvent(QDragEnterEvent *event) {
    if (event->mimeData()->hasText()) {
        event->acceptProposedAction();
    }
}

void MainWindow::dropEvent(QDropEvent *event) {
    QString droppedText = event->mimeData()->text().trimmed();
    if (droppedText.startsWith("http")) {
        ui->urlLineEdit->setText(droppedText);
        on_fetchFormats_clicked(); // تحليل الجودة تلقائيًا
    }
}
